"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GameBase = void 0;
class GameBase {
    constructor() {
        this._step = 0;
        this._actionStep = 0;
        this._hasFinished = false;
        this._gameSettings = this.getGameSettings();
        this._gaSettings = this.getGaSettings();
    }
    get step() {
        return this._step;
    }
    get actionStep() {
        return this._actionStep;
    }
    get gameSettings() {
        return this._gameSettings;
    }
    get gaSettings() {
        return this._gaSettings;
    }
    get stepLimit() {
        var _a;
        // eslint-disable-next-line no-magic-numbers
        return (_a = this.gameSettings.stepLimit) !== null && _a !== void 0 ? _a : (this.gameSettings.actionLimit * 10);
    }
    get hasReached() {
        return this._hasFinished || this.actionStep >= this.gameSettings.actionLimit || this.step >= this.stepLimit;
    }
    onFinished() {
        this._hasFinished = true;
    }
    action(index) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.hasReached) {
                throw new Error('The step has reached to limit');
            }
            this._step++;
            this._actionStep++;
            yield this.performAction(index);
        });
    }
    perceive(index) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.hasReached) {
                throw new Error('The step has reached to limit');
            }
            this._step++;
            return this.performPerceive(index);
        });
    }
    actionExpression(index) {
        return `行動${index}`;
    }
    perceiveExpression(index) {
        return `知覚${index}`;
    }
    getCorrectionItemFitness() {
        // eslint-disable-next-line no-magic-numbers
        return 0;
    }
    getFitness() {
        const fitness = this.performGetFitness();
        if (fitness >= 1) { // eslint-disable-line no-magic-numbers
            return fitness + 1 / (this.actionStep + this.step); // eslint-disable-line no-magic-numbers
        }
        return Math.max(0, // eslint-disable-line no-magic-numbers
        fitness + this.getCorrectionItemFitness());
    }
    static getCanvas(target) {
        if (typeof target === 'string') {
            return document.getElementById(target);
        }
        return target;
    }
    draw(target) {
        const canvas = GameBase.getCanvas(target);
        if (!canvas) {
            return;
        }
        canvas.width = this.gameSettings.width;
        canvas.height = this.gameSettings.height;
        const style = canvas.style;
        style.width = `${this.gameSettings.width}px`;
        style.height = `${this.gameSettings.height}px`;
        style.backgroundColor = this.getBackgroundColor();
        const context = canvas.getContext('2d');
        if (!context) {
            return;
        }
        // eslint-disable-next-line no-magic-numbers
        const imageData = context.getImageData(0, 0, canvas.width, canvas.height);
        const pixels = imageData.data;
        this.preDrawPixels(pixels);
        this.performDrawPixels(pixels);
        // eslint-disable-next-line no-magic-numbers
        context.putImageData(imageData, 0, 0);
        this.performDraw(context);
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    preDrawPixels(pixels) {
        //
    }
    getBackgroundColor() {
        return '#FFF';
    }
    // eslint-disable-next-line id-length,@typescript-eslint/no-unused-vars
    getPixelColor(x, y) {
        return undefined;
    }
    parseColors(colors) {
        if (colors.length === 1) { // eslint-disable-line no-magic-numbers
            return {
                red: colors[0],
                green: colors[0],
                blue: colors[0],
                alpha: 255,
            };
        }
        else if (colors.length === 2) { // eslint-disable-line no-magic-numbers
            return {
                red: colors[0],
                green: colors[0],
                blue: colors[0],
                alpha: colors[1],
            };
        }
        else if (colors.length === 3) { // eslint-disable-line no-magic-numbers
            return {
                red: colors[0],
                green: colors[1],
                blue: colors[2],
                alpha: 255,
            };
        }
        return {
            red: colors[0],
            green: colors[1],
            blue: colors[2],
            alpha: colors[3],
        };
    }
    performDrawPixels(pixels) {
        // eslint-disable-next-line no-magic-numbers,id-length
        for (let y = this.gameSettings.height; --y >= 0;) {
            // eslint-disable-next-line no-magic-numbers,id-length
            for (let x = this.gameSettings.width; --x >= 0;) {
                const index = (y * this.gameSettings.width + x) * 4; // eslint-disable-line no-magic-numbers
                const colors = this.getPixelColor(x, y);
                if (!colors) {
                    continue;
                }
                const { red, green, blue, alpha } = this.parseColors(colors);
                pixels[index] = red;
                pixels[index + 1] = green; // eslint-disable-line no-magic-numbers
                pixels[index + 2] = blue; // eslint-disable-line no-magic-numbers
                pixels[index + 3] = alpha; // eslint-disable-line no-magic-numbers
            }
        }
    }
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    performDraw(context) {
        //
    }
}
exports.GameBase = GameBase;
//# sourceMappingURL=GameBase.js.map